@@ -21,6 +21,10 @@
 #include <unistd.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+#if defined(__clang__)
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdocumentation"
+#endif
 #include <linux/if_vlan.h>
 #include <linux/if_bonding.h>
 #include <linux/if_bridge.h>
@@ -28,6 +32,9 @@
 #include <linux/sockios.h>
 #include <linux/if_packet.h>
 #include <linux/ethtool.h>
+#if defined(__clang__)
+#pragma clang diagnostic pop
+#endif
 
 #define SYSFS_PATH_MAX 256
 #define MAX_PORTS 1024
@@ -104,12 +111,12 @@ static struct lldpd_ops eth_ops = {
 	.cleanup = iflinux_eth_close,
 };
 
+#ifdef ENABLE_OLDIES
 static int
 old_iflinux_is_bridge(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *iface)
 {
-#ifdef ENABLE_OLDIES
 	int j;
 	int ifptindices[MAX_PORTS] = {};
 	unsigned long args2[4] = {
@@ -147,16 +154,15 @@ old_iflinux_is_bridge(struct lldpd *cfg,
 		}
 	}
 	return 1;
-#else
-	return 0;
-#endif
 }
+#endif
 
 static int
 iflinux_is_bridge(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *iface)
 {
+#ifdef ENABLE_OLDIES
 	struct interfaces_device *port;
 	char path[SYSFS_PATH_MAX];
 	int f;
@@ -186,6 +192,9 @@ iflinux_is_bridge(struct lldpd *cfg,
 	}
 
 	return 1;
+#else
+	return 0;
+#endif
 }
 
 static int
@@ -193,6 +202,7 @@ iflinux_is_vlan(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *iface)
 {
+#ifdef ENABLE_OLDIES
 	struct vlan_ioctl_args ifv = {};
 	ifv.cmd = GET_VLAN_REALDEV_NAME_CMD;
 	strlcpy(ifv.device1, iface->name, sizeof(ifv.device1));
@@ -221,6 +231,7 @@ iflinux_is_vlan(struct lldpd *cfg,
 		iface->vlanid = ifv.u.VID;
 		return 1;
 	}
+#endif
 	return 0;
 }
 
@@ -229,6 +240,7 @@ iflinux_is_bond(struct lldpd *cfg,
     struct interfaces_device_list *interfaces,
     struct interfaces_device *master)
 {
+#ifdef ENABLE_OLDIES
 	/* Shortcut if we detect the new team driver. Upper and lower links
 	 * should already be set with netlink in this case.  */
 	if (master->driver && !strcmp(master->driver, "team")) {
@@ -261,6 +273,7 @@ iflinux_is_bond(struct lldpd *cfg,
 		}
 		return 1;
 	}
+#endif
 	return 0;
 }
 
